"""
Settings for the apigateway app. See the README.md for more detail.

These should be reasonable defaults - override (or pull from env) as necessary.
"""

# apigateway configuration

# Disable middleware. For local testing - you can have the middleware in place
# but not use it and use Django's built-in users instead.
MITOL_APIGATEWAY_DISABLE_MIDDLEWARE = False

# The header that contains the user data from the upstream API gateway.
MITOL_APIGATEWAY_USERINFO_HEADER_NAME = "HTTP_X_USERINFO"

# The header that contains the value we want for user ID.
# This should match with USERNAME_FIELD in the user model.
MITOL_APIGATEWAY_USERINFO_ID_FIELD = "sub"

# Maps user data from the upstream API gateway to the user model(s)
MITOL_APIGATEWAY_USERINFO_MODEL_MAP = {
    # Mappings to the user model.
    "user_fields": {
        # Keys are data returned from the API gateway.
        # Values are the user object field name.
        "preferred_username": "username",
        "email": "email",
        "sub": "global_id",
        "name": "name",
    },
    # Additional models to map in.
    # Key is the model name, then a list of tuples of header field name, model
    # field name, and default. The FK for the related user should be "user".
    "additional_models": {
        # Sample:
        # "users.UserProfile": [
        #     ("email_optin", "email_optin", False),  # noqa: ERA001
        #     ("country_code", "country_code", ""),  # noqa: ERA001
        # ],
        # ..then add additional ones here if needed
    },
}

# Set to True to create users that we see but aren't aware of.
# Set to False if you're managing that elsewhere (like with social-auth).
MITOL_APIGATEWAY_USERINFO_CREATE = True

# Set to True to update users we've seen before. If you set this to False, make
# sure there's a backchannel way to update the user data (SCIM, etc) or user
# info will fall out of sync with the IdP pretty quickly.
MITOL_APIGATEWAY_USERINFO_UPDATE = True

# This is the name of the field used to lookup the user
MITOL_APIGATEWAY_USER_LOOKUP_FIELD = "global_id"

# URL configuation

# Set to the URL that APISIX uses for logout.
MITOL_APIGATEWAY_LOGOUT_URL = "/logout"

# Set to the default URL the user should be sent to when logging out.
# If there's no redirect URL specified otherwise, the user gets sent here.
MITOL_APIGATEWAY_DEFAULT_POST_LOGOUT_DEST = "/app"

# Set to the list of hosts the app is allowed to redirect to.
MITOL_APIGATEWAY_ALLOWED_REDIRECT_HOSTS = [
    "localhost",
]



"""URL routes for the apigateway app. Mostly for testing."""

from django.urls import path

from mitol.apigateway.views import ApiGatewayLogoutView

urlpatterns = [
    path(
        "applogout/",
        ApiGatewayLogoutView.as_view(),
        name="logout",
    ),
]



"""API Gateway app AppConfig"""

import os

from django.apps import AppConfig


class ApigatewayApp(AppConfig):
    """Default configuration for the Apigateway app"""

    name = "mitol.apigateway"
    label = "apigateway"
    verbose_name = "apigateway"

    # necessary because this is a namespaced app
    path = os.path.dirname(os.path.abspath(__file__))  # noqa: PTH100, PTH120


class TransitionalApigatewayApp(AppConfig):
    """AppConfig for transitioning a project with an existing 'Apigateway' app"""

    name = "mitol.apigateway"
    label = "transitional_apigateway"
    verbose_name = "apigateway"

    # necessary because this is a namespaced app
    path = os.path.dirname(os.path.abspath(__file__))  # noqa: PTH100, PTH120



"""Promote/demote a user."""

from django.contrib.auth import get_user_model
from django.core.management import BaseCommand, CommandError


class Command(BaseCommand):
    """Promote/demote a user."""

    help = "Promote/demote a user."

    def add_arguments(self, parser):
        """Add arguments to the command."""

        subparsers = parser.add_subparsers(
            title="Action",
            dest="subcommand",
            required=True,
        )

        promote_parser = subparsers.add_parser("promote", help="Promote a user.")
        demote_parser = subparsers.add_parser("demote", help="Demote a user.")

        promote_parser.add_argument(
            "--superuser",
            action="store_true",
            help="Promote the user to superuser.",
        )
        promote_parser.add_argument(
            "--email",
            type=str,
            help="The email of the user to promote/demote.",
            required=True,
        )

        demote_parser.add_argument(
            "--staff",
            action="store_true",
            help="Demote the user from superuser to staff.",
        )
        demote_parser.add_argument(
            "--email",
            type=str,
            help="The email of the user to promote/demote.",
            required=True,
        )

    def _get_user(self, email: str) -> get_user_model:
        """Get a user by email."""

        User = get_user_model()

        try:
            user = User.objects.get(email=email)
        except User.DoesNotExist as exc:
            errmsg = f"User with email {email} does not exist."
            raise CommandError(errmsg) from exc

        return user

    def handle_promote(self, *args, **options) -> None:  # noqa: ARG002
        """Handle user promotion."""

        email = options["email"]
        superuser = options["superuser"]

        user = self._get_user(email)

        user.is_staff = True
        user.is_superuser = superuser

        user.save()

        superuser_str = " to superuser" if superuser else ""
        self.stdout.write(
            self.style.SUCCESS(
                f"Successfully promoted user {user.email}{superuser_str}."
            )
        )

    def handle_demote(self, *args, **options) -> None:  # noqa: ARG002
        """Handle user demotion."""

        email = options["email"]
        staff = options["staff"]

        user = self._get_user(email)

        user.is_staff = staff and user.is_superuser
        user.is_superuser = False

        user.save()

        staff_str = " from superuser to staff" if staff else ""
        self.stdout.write(
            self.style.SUCCESS(f"Successfully demoted user {user.email}{staff_str}.")
        )

    def handle(self, **options) -> None:
        """Handle the command."""

        subcommand = options["subcommand"]

        if subcommand == "promote":
            self.handle_promote(**options)
        elif subcommand == "demote":
            self.handle_demote(**options)
        else:
            errmsg = "You must specify promote or demote."
            raise CommandError(errmsg)
